#include "squidwall.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * some simple debugging can be made
 */

#if DEBUG_LEVEL
int debug_level=DEBUG_LEVEL;

void debug(int level, char *msg)
{
	if (!isset(debug_level, level)) return;

	log_headline("debug"); log_s(msg); log_s("\n");

	return;
}

void debug_ulong(int level, char *msg, unsigned long num)
{
	if (!isset(debug_level, level)) return;

	log_headline("debug");
	log_s(msg); log_ulong(num); log_s("\n");

	return;
}

void debug_long(int level, char *msg, long num)
{
	if (!isset(debug_level, level)) return;

	log_headline("debug");
	log_s(msg); log_long(num); log_s("\n");

	return;
}

void debug_s(int level, char *s)
{
	debug_s5(level,s,0,0,0,0);

	return;
}

void debug_s2(int level, char *s1, char *s2)
{
	debug_s5(level,s1,s2,0,0,0);

	return;
}

void debug_s3(int level, char *s1, char *s2, char *s3)
{
	debug_s5(level,s1,s2,s3,0,0);

	return;
}

void debug_s4(int level, char *s1, char *s2, char *s3, char *s4)
{
	debug_s5(level,s1,s2,s3,s4,0);

	return;
}

void debug_s5(int level, char *s1, char *s2, char *s3, char *s4, char *s5)
{
	if (!isset(debug_level, level)) return;

	log_headline("debug");
	if (s1) log_s(s1);
	if (s2) log_s(s2);
	if (s3) log_s(s3);
	if (s4) log_s(s4);
	if (s5) log_s(s5);
	log_s("\n");

	return;
}

void debug_sa(int level, char *msg, stralloc *sa)
{
	if (!isset(debug_level, level)) return;

	log_headline("debug");
	log_s(msg); log_sa(sa); log_s("\n");

	return;
}

void debug_todo(char *msg, int todo)
{
	stralloc sa;

	if (!isset(debug_level, DEBUG_SQUIDWALL)) return;

	log_headline("debug");
	log_s("todo: ");
	log_s(msg);
	stralloc_init(&sa);
	if (isset(todo, TODO_PHRASE))  if (!stralloc_cats(&sa, " TODO_PHRASE")) die_nomem();
	if (isset(todo, TODO_CONT  ))  if (!stralloc_cats(&sa, " TODO_CONT")) die_nomem();
	if (isset(todo, TODO_CONTr ))  if (!stralloc_cats(&sa, " TODO_CONTr")) die_nomem();
	if (isset(todo, TODO_URL   ))  if (!stralloc_cats(&sa, " TODO_URL")) die_nomem();
	if (isset(todo, TODO_URLr  ))  if (!stralloc_cats(&sa, " TODO_URLr")) die_nomem();
	if (isset(todo, TODO_URLmd5))  if (!stralloc_cats(&sa, " TODO_URLmd5")) die_nomem();
	if (isset(todo, TODO_URLsha1)) if (!stralloc_cats(&sa, " TODO_URLsha1")) die_nomem();
	if (isset(todo, TODO_BAN   ))  if (!stralloc_cats(&sa, " TODO_BAN")) die_nomem();
	if (isset(todo, TODO_BANr  ))  if (!stralloc_cats(&sa, " TODO_BANr")) die_nomem();
	if (isset(todo, TODO_BANc  ))  if (!stralloc_cats(&sa, " TODO_BANc")) die_nomem();
	if (isset(todo, TODO_WLST  ))  if (!stralloc_cats(&sa, " TODO_WLST")) die_nomem();
	if (isset(todo, TODO_WLSTr ))  if (!stralloc_cats(&sa, " TODO_WLSTr")) die_nomem();
	if (isset(todo, TODO_VIRUS ))  if (!stralloc_cats(&sa, " TODO_VIRUS")) die_nomem();
	log_sa(&sa);
	log_s("\n");
	stralloc_free(&sa);

	return;
}

void debug_flags(char *msg, int flags)
{
	stralloc sa;

	if (!isset(debug_level, DEBUG_SQUIDWALL)) return;

	log_headline("debug");
	log_s("flags: ");
	log_s(msg);
	stralloc_init(&sa);
	if (isset(flags, ACL_ALLOW ))  if (!stralloc_cats(&sa, " ACL_ALLOW")) die_nomem();
	if (isset(flags, ACL_FILTER))  if (!stralloc_cats(&sa, " ACL_FILTER")) die_nomem();
	if (isset(flags, ACL_DENY  ))  if (!stralloc_cats(&sa, " ACL_DENY")) die_nomem();
	if (isset(flags, ACL_REDIR ))  if (!stralloc_cats(&sa, " ACL_REDIR")) die_nomem();
	log_sa(&sa);
	log_s("\n");
	stralloc_free(&sa);

	return;
}

void debug_controls(void)
{
	if (!isset(debug_level, DEBUG_READ_CONTROLS)) return;
#define log_s3(s1,s2,s3) { log_s(s1); log_s(s2); log_s(s3); }

	log_headline("debug");log_s("cache:           "); log_ulong(cache);log_s("\n");
	log_headline("debug");log_s("globals:         "); log_ulong(globals);log_s("\n");
	log_headline("debug");log_s("maxsize:         "); log_ulong(maxsize);log_s("\n");
	log_headline("debug");log_s("memcache:        "); log_ulong(memcache);log_s("\n");
#if DEBUG_LEVEL
	log_headline("debug");log_s("debug_level:     "); log_ulong(debug_level);log_s("\n");
#endif
	log_headline("debug");log_s("pfilter_max:     "); log_ulong(pfilter_max);log_s("\n");
	log_headline("debug");log_s("pfilter_default: "); log_ulong(pfilter_default);log_s("\n");
	log_headline("debug");log_s("sqlite_timeout:  "); log_ulong(sqlite_timeout);log_s("\n");

	log_headline("debug");log_s("proxy            "); log_s3("[", proxy, "]"); log_s("\n");
	log_headline("debug");log_s("logfile          "); log_s3("[", logfile, "]"); log_s("\n");
	log_headline("debug");log_s("host_dir         "); log_s3("[", host_dir, "]"); log_s("\n");
	log_headline("debug");log_s("user_dir         "); log_s3("[", user_dir, "]"); log_s("\n");
	log_headline("debug");log_s("cache_dir        "); log_s3("[", cache_dir, "]"); log_s("\n");
	log_headline("debug");log_s("redir_dir        "); log_s3("[", redir_dir, "]"); log_s("\n");

	log_headline("debug");log_s("clamav_tcp       "); log_s3("[", clamav_tcp, "]"); log_s("\n");
	log_headline("debug");log_s("clamav_socket    "); log_s3("[", clamav_socket, "]"); log_s("\n");

	log_headline("debug");log_s("redir_badcont    "); log_s3("[", redir_badcont, "]"); log_s("\n");
	log_headline("debug");log_s("redir_badhost    "); log_s3("[", redir_badhost, "]"); log_s("\n");
	log_headline("debug");log_s("redir_badurl     "); log_s3("[", redir_badurl, "]"); log_s("\n");
	log_headline("debug");log_s("redir_baduser    "); log_s3("[", redir_baduser, "]"); log_s("\n");
	log_headline("debug");log_s("redir_bannerfound"); log_s3("[", redir_bannerfound, "]"); log_s("\n");
	log_headline("debug");log_s("redir_safesite   "); log_s3("[", redir_safesite, "]"); log_s("\n");
	log_headline("debug");log_s("redir_virusfound "); log_s3("[", redir_virusfound, "]"); log_s("\n");
	log_headline("debug");log_s("redir_webinterfa:"); log_s3("[", redir_webinterface, "]"); log_s("\n");

	log_headline("debug");log_s("bfilter_dir      "); log_s3("[", bfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("bcfilter_dir     "); log_s3("[", bcfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("brfilter_dir     "); log_s3("[", brfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("cfilter_dir      "); log_s3("[", cfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("crfilter_dir     "); log_s3("[", crfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("pfilter_dir      "); log_s3("[", pfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("ufilter_dir      "); log_s3("[", ufilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("ufilter_md5_dir  "); log_s3("[", ufilter_md5_dir, "]"); log_s("\n");
	log_headline("debug");log_s("ufilter_sha1_dir "); log_s3("[", ufilter_sha1_dir, "]"); log_s("\n");
	log_headline("debug");log_s("urfilter_dir     "); log_s3("[", urfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("wfilter_dir      "); log_s3("[", wfilter_dir, "]"); log_s("\n");
	log_headline("debug");log_s("wrfilter_dir     "); log_s3("[", wrfilter_dir, "]"); log_s("\n");

	return;
}

#endif
