#include "squidwall.h"
#include "timestamp.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * logging functions
 */

/* buffer for the logfile */
static char log_BUF[BUFSIZE];
static buffer buffer_log;

/* pid of _this_ redirector process */
static pid_t pid;

void log_init(int reload)
{
	static int64 fd_log=-1;
	static int initdone=0;

	if (!reload) return;
	if (!logfile) return;

	if (fd_log != -1) close(fd_log);
	if (!io_appendfile(&fd_log, logfile)) panic("could not open logfile");
	buffer_init(&buffer_log, (int(*)())write, fd_log, log_BUF, sizeof log_BUF);

	if (initdone) return;
	pid=getpid();
	initdone=1;

	return;
}

void log_timestamp(void)
{
	char s[TIMESTAMP];

	log_init(0);
	timestamp(s);
	buffer_put(&buffer_log, s, TIMESTAMP);
	buffer_puts(&buffer_log, " ");

	return;
}

void log_s(char *s)
{
	log_init(0);
	buffer_puts(&buffer_log, s);

	return;
}

void log_sa(stralloc *sa)
{
	log_init(0);
	buffer_putsa(&buffer_log, sa);

	return;
}

void log_flush(void)
{
	log_init(0);
	buffer_flush(&buffer_log);

	return;
}

void log_ulong(unsigned long n)
{
	char s[FMT_ULONG];

	log_init(0);
	s[fmt_ulong(s, n)]=0;
	buffer_puts(&buffer_log, s);

	return;
}

#if DEBUG_LEVEL
void log_long(unsigned long n)
{
	char s[FMT_LONG];

	log_init(0);
	s[fmt_long(s, n)]=0;
	buffer_puts(&buffer_log, s);

	return;
}
#endif

void log_pid(void)
{
	log_init(0);
	log_ulong(pid);

	return;
}

void log_headline(char *headline)
{
	int l=str_len(headline);

	log_init(0);
	log_timestamp();
	log_s(headline);
	log_s("[");
	log_pid();
	log_s("]:");
	while (l<10) { log_s(" "); l++; }

	return;
}
