#ifndef SW_PRAMS_H
#define SW_PRAMS_H

/**
 * all these values can be overridden by environment variables!
 */

/* this logfile will show you the denied users/hosts/pages */
#define LOGFILE          "/var/log/squid/squidwall.log"

/* here you can define your default log behaviour */
#define LOG_STARTUP_INFO        1
#define LOG_SQUID_REQUEST       2
#define LOG_SQUIDWALL_RESPONSE  4
#define LOG_CLAMAV_STATUS       8
#define LOG_MEMCACHE_HITS      16

#define LOGTODO                (1|2|4|8)

/**
 * ip:port of your squid cache
 *
 * warning: this cache should not re-redirect to the redirector :)
 */
#define PROXY            "127.0.0.1:3128"

/* ip:port of clamd in TCP/IP mode */
#define CLAMAV_TCP       "127.0.0.1:3310"

/* /some/path/to/clamd.sock of clamd in local socket mode */
#define CLAMAV_SOCKET    "/var/lib/clamav/clamd.sock"

/**
 * the default size in bytes, which will be passed through the content
 * filter and the clamav stream for antivirus checking
 */
#define MAXSIZE          60000

/**
 * how many url should be remebered in memory ... can speed up things :)
 */
#define MEMCACHE         100

/* phrase filter */
#define PFILTER_MAX      100 /* deadline for the pfilters, variable: 0..255 */
#define PFILTER_DEFAULT  25  /* for unacceptable values in configuration */

/* default timeout, for sqlite3 busy requests */
#define SQLITE_TIMEOUT   300

/* user / host acl's and redirects */
#define CACHE_DIR         "/var/cache/squidwall"
#define CONFIG_DIR        "/etc/squidwall"
#define HOST_DIR          CONFIG_DIR "/hosts"
#define USER_DIR          CONFIG_DIR "/users"
#define CONTROL_DIR       CONFIG_DIR "/control"
#define REDIR_DIR         CONFIG_DIR "/redirects"
#define GLOBAL_DIR        CONFIG_DIR "/globals"

/* content filters */
#define PFILTER_DIR       CONFIG_DIR "/filters/pfilter"
#define CFILTER_DIR       CONFIG_DIR "/filters/cfilter"
#define CRFILTER_DIR      CONFIG_DIR "/filters/crfilter"
#define BCFILTER_DIR      CONFIG_DIR "/filters/bcfilter"

/* url filters */
#define BFILTER_DIR       CONFIG_DIR "/filters/bfilter"
#define BRFILTER_DIR      CONFIG_DIR "/filters/brfilter"
#define UFILTER_DIR       CONFIG_DIR "/filters/ufilter"
#define URFILTER_DIR      CONFIG_DIR "/filters/urfilter"
#define WFILTER_DIR       CONFIG_DIR "/filters/wfilter"
#define WRFILTER_DIR      CONFIG_DIR "/filters/wrfilter"
#define UFILTER_MD5_DIR   CONFIG_DIR "/filters/ufilter_md5"
#define UFILTER_SHA1_DIR  CONFIG_DIR "/filters/ufilter_sha1"

/* bayes learning filters */
#define BAYES_OSBF_DIR    CONFIG_DIR "/filters/bayes_osbf"

/* you can configure your deny page here */
#define REDIR_SERVER       "http://www.example.org"
#define REDIR_NETWORK      "x"
#define REDIR_BADURL       REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=url&"
#define REDIR_BADCONT      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=cont&"
#define REDIR_BADHOST      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=host&"
#define REDIR_BADUSER      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=user&"
#define REDIR_VIRUSFOUND   REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=virus&"
#define REDIR_BANNERFOUND  REDIR_SERVER "/inet/img/t.png"

/* this url nerver redirected! */
#define REDIR_SAFESITE     REDIR_SERVER

/* this url is used for configuration via http://squidwall */
#define REDIR_WEBINTERFACE REDIR_SERVER "/inet"

/* debugging of some functions / files */
#define DEBUG_SQUIDWALL        1
#define DEBUG_READ_ACLS        2
#define DEBUG_READ_FILTERS     4
#define DEBUG_READ_CONTROLS    8
#define DEBUG_CHECK_ACLS      16
#define DEBUG_CHECK_BANNER    32
#define DEBUG_CHECK_WLIST     64
#define DEBUG_CHECK_BLIST    128
#define DEBUG_CHECK_CONT     256
#define DEBUG_CHECK_REDIR    512
#define DEBUG_CHECK_VIRUS   1024
#define DEBUG_CHECK_GLOBALS 2048
#define DEBUG_CACHE_CONTENT 4096
#define DEBUG_SQLITE        8192
#define DEBUG_MEMCACHE     16384

#define DEBUG_ALL   (2*16384 -1)
//#define DEBUG_LEVEL DEBUG_ALL

#endif
