#include "squidwall.h"

/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

stralloc sa_proxy;
stralloc sa_logfile;
stralloc sa_logtodo;
stralloc sa_host_dir;
stralloc sa_user_dir;
stralloc sa_cache_dir;
stralloc sa_redir_dir;
stralloc sa_globals_dir;
stralloc sa_clamav_tcp;
stralloc sa_clamav_socket;

stralloc sa_redir_badcont;
stralloc sa_redir_badhost;
stralloc sa_redir_badurl;
stralloc sa_redir_baduser;
stralloc sa_redir_bannerfound;
stralloc sa_redir_safesite;
stralloc sa_redir_virusfound;
stralloc sa_redir_webinterface;

stralloc sa_bcfilter_dir;
stralloc sa_bfilter_dir;
stralloc sa_brfilter_dir;
stralloc sa_cfilter_dir;
stralloc sa_crfilter_dir;
stralloc sa_pfilter_dir;
stralloc sa_ufilter_dir;
stralloc sa_ufilter_md5_dir;
stralloc sa_ufilter_sha1_dir;
stralloc sa_urfilter_dir;
stralloc sa_wfilter_dir;
stralloc sa_wrfilter_dir;

static void read_controls_init(void);
static int read_controls_int(char *path, char *file, int *oldval);
static int read_controls_str(char *path, char *file, stralloc *val, char *old);

static void read_controls_init(void)
{
	static int done=0;

	if (done) return;
	done=1;

	stralloc_init(&sa_proxy);
	stralloc_init(&sa_logfile);
	stralloc_init(&sa_logtodo);
	stralloc_init(&sa_host_dir);
	stralloc_init(&sa_user_dir);
	stralloc_init(&sa_cache_dir);
	stralloc_init(&sa_redir_dir);
	stralloc_init(&sa_globals_dir);
	stralloc_init(&sa_clamav_tcp);
	stralloc_init(&sa_clamav_socket);

	stralloc_init(&sa_redir_badcont);
	stralloc_init(&sa_redir_badhost);
	stralloc_init(&sa_redir_badurl);
	stralloc_init(&sa_redir_baduser);
	stralloc_init(&sa_redir_bannerfound);
	stralloc_init(&sa_redir_safesite);
	stralloc_init(&sa_redir_virusfound);
	stralloc_init(&sa_redir_webinterface);

	stralloc_init(&sa_bcfilter_dir);
	stralloc_init(&sa_bfilter_dir);
	stralloc_init(&sa_brfilter_dir);
	stralloc_init(&sa_cfilter_dir);
	stralloc_init(&sa_crfilter_dir);
	stralloc_init(&sa_pfilter_dir);
	stralloc_init(&sa_ufilter_dir);
	stralloc_init(&sa_ufilter_md5_dir);
	stralloc_init(&sa_ufilter_sha1_dir);
	stralloc_init(&sa_urfilter_dir);
	stralloc_init(&sa_wfilter_dir);
	stralloc_init(&sa_wrfilter_dir);

	return;
}


static int read_controls_int(char *path, char *file, int *old)
{
	stralloc sa;
	int r;

	stralloc_init(&sa);

	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/")) die_nomem();
	if (!stralloc_cats(&sa, file)) die_nomem();
	sa.s[sa.len]=0;

	r=control_readint(sa.s, old);
	stralloc_free(&sa);

	if (r == 1) return 1;
	return 0;
}


static int read_controls_str(char *path, char *file, stralloc *val, char *old)
{
	stralloc sa;
	int r;

	/* initialize all control stralloc's */
	read_controls_init();

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/")) die_nomem();
	if (!stralloc_cats(&sa, file)) die_nomem();
	sa.s[sa.len]=0;

	r=control_readline(sa.s, val, old);
	stralloc_free(&sa);

	if (r == 1) return 1;
	return 0;
}


/**
 * read_controls - re-read squidwall variables
 */
int read_controls(char *path, time_t *mtime)
{
	int r, changes=0;
	struct stat s;
	stralloc sa;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/!")) die_nomem();
	if (!stralloc_0(&sa)) die_nomem();

	/* if stat fails, we return 0 (meaning no change) */
	if (stat(sa.s, &s) == -1) {
		stralloc_free(&sa);
		return 0;
	}
	stralloc_free(&sa);

	if (*mtime == s.st_mtime) {
		return 0; /* was not modified... */
	}

	/* was modified... */
	*mtime = s.st_mtime;

	r=read_controls_int(path, "cache", &cache);
	if (r) changes++;

	r=read_controls_int(path, "globals", &globals);
	if (r) changes++;

	r=read_controls_int(path, "maxsize", &maxsize);
	if (r) changes++;

	r=read_controls_int(path, "logtodo", &logtodo);
	if (r) changes++;

	r=read_controls_int(path, "memcache", &memcache);
	if (r) changes++;

#if DEBUG_LEVEL
	r=read_controls_int(path, "debug_level", &debug_level);
	if (r) changes++;
#endif

	r=read_controls_int(path, "pfilter_max", &pfilter_max);
	if (r) changes++;

	r=read_controls_int(path, "pfilter_default", &pfilter_default);
	if (r) changes++;

	r=read_controls_int(path, "sqlite_timeout", &sqlite_timeout);
	if (r) changes++;

	r = read_controls_str(path, "proxy", &sa_proxy, proxy);
	if (r) { changes++; proxy=sa_proxy.s; }
	r=ip4_scan(proxy, (char*)&proxy_ip);
	if (!r) panic("can't get ip of proxy!");
	r=scan_uint(proxy+r+1, &proxy_port);
	if (!r) panic("can't get port of proxy!");

	r = read_controls_str(path, "logfile",
		&sa_logfile, logfile);
	if (r) { changes++; logfile=sa_logfile.s; log_init(1); }

	r = read_controls_str(path, "host_dir",
		&sa_host_dir, host_dir);
	if (r) { changes++; host_dir=sa_host_dir.s; }

	r = read_controls_str(path, "user_dir",
		&sa_user_dir, user_dir);
	if (r) { changes++; user_dir=sa_user_dir.s; }

	r = read_controls_str(path, "cache_dir",
		&sa_cache_dir, cache_dir);
	if (r) { changes++; cache_dir=sa_cache_dir.s; }

	r = read_controls_str(path, "redir_dir",
		&sa_redir_dir, redir_dir);
	if (r) { changes++; redir_dir=sa_redir_dir.s; }

	r = read_controls_str(path, "global_dir",
		&sa_globals_dir, global_dir);
	if (r) { changes++; global_dir=sa_globals_dir.s; }

	r = read_controls_str(path, "clamav_tcp",
		&sa_clamav_tcp, clamav_tcp);
	if (r) { changes++; clamav_tcp=sa_clamav_tcp.s; }
	r=ip4_scan(clamav_tcp, (char*)&clamd_ip);
	if (!r) panic("can't get ip of clamav daemon!");
	r=scan_uint(clamav_tcp+r+1, &clamd_port);
	if (!r) panic("can't get port of clamav daemon!");

	r = read_controls_str(path, "clamav_socket",
		&sa_clamav_socket, clamav_socket);
	if (r) { changes++; clamav_socket=sa_clamav_socket.s; }

	r = read_controls_str(path, "redir_badcont",
		&sa_redir_badcont, redir_badcont);
	if (r) { changes++; redir_badcont=sa_redir_badcont.s; }

	r = read_controls_str(path, "redir_badhost",
		&sa_redir_badhost, redir_badhost);
	if (r) { changes++; redir_badhost=sa_redir_badhost.s; }

	r = read_controls_str(path, "redir_badurl",
		&sa_redir_badurl, redir_badurl);
	if (r) { changes++; redir_badurl=sa_redir_badurl.s; }

	r = read_controls_str(path, "redir_baduser",
		&sa_redir_baduser, redir_baduser);
	if (r) { changes++; redir_baduser=sa_redir_baduser.s; }

	r = read_controls_str(path, "redir_bannerfound",
		&sa_redir_bannerfound, redir_bannerfound);
	if (r) { changes++; redir_bannerfound=sa_redir_bannerfound.s; }

	r = read_controls_str(path, "redir_safesite",
		&sa_redir_safesite, redir_safesite);
	if (r) { changes++; redir_safesite=sa_redir_safesite.s; }

	r = read_controls_str(path, "redir_virusfound"
		, &sa_redir_virusfound, redir_virusfound);
	if (r) { changes++; redir_virusfound=sa_redir_virusfound.s; }

	r = read_controls_str(path, "redir_webinterface",
		&sa_redir_webinterface, redir_webinterface);
	if (r) { changes++; redir_webinterface=sa_redir_webinterface.s; }

	r = read_controls_str(path, "bfilter_dir",
		&sa_bfilter_dir, bfilter_dir);
	if (r) { changes++; bfilter_dir=sa_bfilter_dir.s; }

	r = read_controls_str(path, "bcfilter_dir",
		&sa_bcfilter_dir, bcfilter_dir);
	if (r) { changes++; bcfilter_dir=sa_bcfilter_dir.s; }

	r = read_controls_str(path, "brfilter_dir",
		&sa_brfilter_dir, brfilter_dir);
	if (r) { changes++; brfilter_dir=sa_brfilter_dir.s; }

	r = read_controls_str(path, "cfilter_dir",
		&sa_cfilter_dir, cfilter_dir);
	if (r) { changes++; cfilter_dir=sa_cfilter_dir.s; }

	r = read_controls_str(path, "crfilter_dir",
		&sa_crfilter_dir, crfilter_dir);
	if (r) { changes++; crfilter_dir=sa_crfilter_dir.s; }

	r = read_controls_str(path, "pfilter_dir",
		&sa_pfilter_dir, pfilter_dir);
	if (r) { changes++; pfilter_dir=sa_pfilter_dir.s; }

	r = read_controls_str(path, "ufilter_dir",
		&sa_ufilter_dir, ufilter_dir);
	if (r) { changes++; ufilter_dir=sa_ufilter_dir.s; }

	r = read_controls_str(path, "ufilter_md5_dir",
		&sa_ufilter_md5_dir, ufilter_md5_dir);
	if (r) { changes++; ufilter_md5_dir=sa_ufilter_md5_dir.s; }

	r = read_controls_str(path, "ufilter_sha1_dir",
		&sa_ufilter_sha1_dir, ufilter_sha1_dir);
	if (r) { changes++; ufilter_sha1_dir=sa_ufilter_sha1_dir.s; }

	r = read_controls_str(path, "urfilter_dir",
		&sa_urfilter_dir, urfilter_dir);
	if (r) { changes++; urfilter_dir=sa_urfilter_dir.s; }

	r = read_controls_str(path, "wfilter_dir",
		&sa_wfilter_dir, wfilter_dir);
	if (r) { changes++; wfilter_dir=sa_wfilter_dir.s; }

	r = read_controls_str(path, "wrfilter_dir",
		&sa_wrfilter_dir, wrfilter_dir);
	if (r) { changes++; wrfilter_dir=sa_wrfilter_dir.s; }

	return changes;
}
